#ifndef __spectrum_inc__
#define __spectrum_inc__

#ifndef __FFTW3_inc__
#include "fftw\\fftw3.inc"
#endif

#ifndef _INC_MATH
#include "..\\include\\math.inc"
#endif

#ifndef __piano_inc__
#include "piano.inc"
#endif

class CFFT
{
	declare CFFT();
	declare _CFFT();
	declare Initialize(HWND hwndDecibels, HWND hwndSpectrum, CPiano *piano);
	declare SetInputSize(int size);
	declare GetInputSize(),int;
	declare SetFrequencyRange(float fMin, float fMax);
	declare GetMinimumFrequency(),float;
	declare GetMaximumFrequency(),float;
	declare SetDataRate(int SamplesPerSec);
	declare SetWindowFunction(FARPROC fn);
	declare SetToneDetectLevel(int level); // positive in decibels
	declare GetToneDetectLevel(),int;
	declare Write(word *samples, int count);
	declare GetSnapshot(float *ppValues, int *ppCount, int *ppSamplesPerSec, int type),BOOL;
	declare SetColors(COLORREF bkColor, COLORREF GridColor, COLORREF PenColor, COLORREF LineColor, COLORREF fontColor);

	declare GetClientRect(RECT *rc);
	declare AllocateBuffers();
	declare FreeBuffers();
	declare OnSize();
	declare CopySpectrumBitmap(HDC hdcTo, HDC hdcFrom);
	declare LevelToDB(float *pSample, float *pValue, float *pDecibels);
	declare UpdateScaling();
	declare UpdateRanges(); // m_i_range_min/max
	declare InitializeGrid();
	declare Draw();
	declare OnPaint();
	declare ComputeMaximumLevel();
	declare f2i(float f),float;
	declare f2x(float f),int;
	declare i2f(int i),float;
	declare i2x(int i),int;
	declare x2f(int x),float;
	declare x2i(int x),int;
	declare GetDetectLevelCoordinate(),int;
	declare DetectLevelFromCoordinate(int y),int;
	declare OnLButtonDown(int x, int y);
	declare OnLButtonUp(int x, int y);
	declare OnMouseMove(int x, int y);
	declare OnKeyDown(int vk);
	declare OnMouseLeave();
	declare OnCaptureChanged();

	COLORREF      m_bkColor;   // 0x008800 ->m_hbrBkgnd
	COLORREF      m_GridColor; // 0x00CC00 ->m_penGrid1, m_penGrid2
	COLORREF      m_PenColor;  // 0xc0FFc0 ->m_penSpectrum
	COLORREF      m_LineColor; // 0x808080 ->m_penDot
	COLORREF      m_fontColor; // 0xFFFFFF ->m_hdcPlotter
	CPiano       *m_piano;
	// spectrum
	HWND          m_hwndSpectrum;
	HDC           m_hdcWindow;
	WNDPROC       m_wndprocSpectrum;  // 
	HDC           m_hdcSpectrum;      //               m_hdcSpectrum - double buffering
	HBITMAP       m_hbmSpectrum;      // selected
	HDC           m_hdcGrid;          //               m_hdcGrid     - grid only
	HBITMAP       m_hbmGrid;          // selected
	HDC           m_hdcPlotter;       //               m_hdcPlotter  - 
	HBITMAP       m_hbmPlotter;       // selected
	SIZE          m_sizeSpectrum;
	HBRUSH        m_hbrBkgnd;         // 
	HPEN          m_penGrid1;         // 
	HPEN          m_penGrid2;         // 
	HPEN          m_penSpectrum;      // 
	HFONT         m_fontSpectrum;     // selected
	// decibels
	HWND          m_hwndDecibels;
	WNDPROC       m_wndprocDecibels;  // 
	HDC           m_hdcDecibels;      // 
	HBITMAP       m_hbmDecibels;      // selected
	SIZE          m_sizeDecibels;
	HFONT         m_fontDecibels;     // 
	HPEN          m_penDot;
	// gdi
	int           m_LevelDivsCount;
	int           m_screenX;
	// scaling
	float        m_FrequencyMin;
	float        m_FrequencyMax;
	float        m_x_scale;       // depends on m_sizeSpectrum.cx, m_i_range_min, m_i_range_max
	float        m_x_shift;       // depends on m_sizeSpectrum.cx, m_i_range_min, m_i_range_max
	float        m_i_range_min;   // depends on m_inputSize, m_SamplesPerSec, m_FrequencyMin, m_FrequencyMax
	float        m_i_range_max;   // depends on m_inputSize, m_SamplesPerSec, m_FrequencyMin, m_FrequencyMax
	float        m_absmax;
	float        m_MinimumPowerDecibels;

	int           m_SamplesPerSec; // UpdateRanges
	int           m_inputSize;     // double count
	int           m_fftSize;       // depends on m_inputSize
	float        *m_samples;
	fftwf_complex *m_fft;

	BOOL          m_fValidOutput;
	FARPROC       m_fnWindowFunction;
	// tone detection
	int           m_DetectLevel; // positive, in decibels

	float        m_CachedMaxGain;
	int           m_CachedBufferSize;
}

#ifndef __spectrum_src__
declare extern MyDrawLine(HDC dc, int x0, int y0, int x1, int y1);
#endif

#endif
